/*
** Restricted Rights Legend
**
** Use, duplication or disclosure by the government is subject to restrictions
** as set forth in subparagraph (c)(1)(ii) of the Rights in Technical Data and
** Computer Software clause at DFARS 252.227-7013.
**
** Copyright (c) 1997 Lockheed Martin C2 Integration Systems
**
**
** File:         Handler.java
**
** Description:  Contains the Handler interface with the following abstract
**               "public" method:
**
**               handler    returns a Reply object.  For use in defining
**                          and registering a KQML unsolicited message
**                          handler for KQML server agents.
**
*/
package kqml;

/**
 * An interface to used to define and register a KQML unsolicited message
 * handler in a KQML server agent.
 *
 * @author Peter Stevens
 * @version $Revision: 1.3 $
 * @see Message
 * @see Reply
 */

public interface Handler
{
    /**
     * Abstract method defining the parameter and return value of a
     * KQML unsolicited message handler.
     *
     * @param msg  an unsolicited Message the handler will process
     * @return     the Reply object (if any) that this message handler generated
     * @see KqmlAgent#registerHandler(java.lang.String, Handler)
     * @see Message
     * @see Reply
     */
    public abstract Reply handler(Message msg);
}
