/* File: ACP3_With_Watcher.java
 * Contains: The main program for bin/acp3-with-watcher
 * Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Created: January 1998
 * Updated: Fri Aug 28 18:14:05 1998 by Jeff Dalton
 * Copyright: (c) 1998, AIAI, University of Edinburgh
 */


package oplan.acp3;

import oplan.util.*;


/** The ACP3 application viewer + Kqml-watcher main program. */

public class ACP3_With_Watcher extends ACP3 {

    public static void main(String[] argv) {

	printGreeting("ACP3 Viewer and KQML Watcher");

	processCommandLineArguments(argv);

	boolean useKqml = Parameters.getBoolean("kqml", true);
	boolean useModel = Parameters.getBoolean("model", true);
	boolean useViewer = Parameters.getBoolean("viewer", true);

	// Create an object to represent the ACP3 application
	agent = new ACP3_With_Watcher();

	// Create the Kqml watcher first, so we don't have to wait
	// before seeing whether it can register as a Kqml agent.
	watcher = new KqmlWatcher(null);
	if (useKqml) {
	    watcher.init();	// set up as Kqml agent
	}

	// Create a viewer
	if (useViewer)
	    viewer = new MainFrame();
	else
	    viewer = new TrivialViewer();

	// Create a process model that sits between the watcher and the viewer
	if (useModel) {
	    model = new ProcessModel();
	    model.initFromFile("ProcessModels/acp3-process.lsp");
	}
	else {
	    model = new TrivialProcessModel();
	}
	model.setViewer(viewer);
	viewer.setProcessModel(model);

	// All command-line arguments should now have been processes.
	Parameters.checkParameterUse();

	// Attach the model to the Kqml watcher
	watcher.setListener(model);

	// Start displaying the monitoring results
	model.sendFullStateDescription();

	// Monitor Kqml
	if (useKqml) {
	    watcher.watcherMainLoop();
	}

    }

    protected void shutdown() {
	if (watcher != null)
	    watcher.shutdown();
	super.shutdown();
    }


}

