/* File: MainFrame.java
 * Contains: ACP3 main viewer frame
 * Author: Austin Tate <a.tate@ed.ac.uk>
 * Created: Wed Jan  7 15:14:27 1998
 * Updated: Mon Aug 24 16:52:33 1998 by Jeff Dalton
 * Copyright: (c) 1998, AIAI, University of Edinburgh
 */

package oplan.acp3;

import java.awt.*;
import java.awt.image.*;
import java.net.*;
import java.util.*;

import oplan.util.*;


class LogoCanvas extends Canvas {
 
   private int canvasWidth;
   private int canvasHeight;
   private Dimension myDimension;
   private Image logo;   
   private Image offsc;
   private Graphics osg;
      
   LogoCanvas(int x, int y, Image logo) {
      super();
      canvasWidth=x;
      canvasHeight=y;
      myDimension=new Dimension(x,y);
      this.logo=logo;
   }
 
   public Dimension minimumSize() {
          return myDimension;
   }
 
   public Dimension preferredSize() {
          return myDimension;
   }
 
   public void update (Graphics g) {
       Debug.noteln("LogoCanvas update called");
       paint(g);
   }
   
   public void paint(Graphics g) {
       if (offsc == null) {
	   offsc = createImage(canvasWidth, canvasHeight);
	   osg=offsc.getGraphics();
	   // once and for all setup of logo, etc into off screen image
	   // image is definitiely available - so size known
	   osg.setColor(Color.gray);
           osg.drawLine(10, 5, (this.getSize().width) - 10, 5);
           osg.setFont(new Font("SansSerif", Font.BOLD, 32));
           osg.drawString("ACP", 20, 32+5);
           osg.setFont(new Font("SansSerif", Font.BOLD, 12));
           osg.drawString("3", 92, 20);
           osg.setFont(new Font("SansSerif", Font.PLAIN, 10));
           osg.drawString("Based on O-Plan Technology", 110, 22);
           osg.drawString("Funded by ARPI", 110, 36);
           osg.drawImage(logo,
                         (this.getSize().width) - (logo.getWidth(this) + 20),
                         10, this);
       }
       g.drawImage(offsc, 0, 0, this);
   }

}

public class MainFrame extends Frame implements ProcessViewer {
    ViewMatrix vm;
    ViewGraph vg;
    InfoFrame info;

    protected ProcessModel model = null;
    
    public Node m_hitNode = null;  // *** added by JL ***

    private Dimension myDimension;
    
    int ScreenNumber = -1;

    private static final int MONITOR_SCREEN = -2; // a special ScreenNumber
    private static final int FROZEN_SCREEN = -3;  // a special ScreenNumber

    private static final String mfLabel="ACP3";
    private static final String infoLabel="ACP3 Information";

    // default frame dimensions and positions
    private static int mfWidth=740, mfHeight=840, mfXPosn=20, mfYPosn=25,
                   infoWidth=350, infoHeight=400, infoXPosn=770, infoYPosn=25;

    // Old defaults
    // private static int mfWidth=720, mfHeight=880, mfXPosn=50, mfYPosn=50,
    //           infoWidth=350, infoHeight=400, infoXPosn=780, infoYPosn=50;

    // Names for Some things we'll need to refer to later on but which
    // are otherwise like variables local to the constructor.  /\/
    Button monitorButton;
    COAMenu mfMenuGraphCOA;

    public MainFrame() {
        super(mfLabel);

	// Process any frame dimension or position parameters
	processGeometryParameters();

        // set up instance variable values
        myDimension=new Dimension(mfWidth,mfHeight);                

        //initialise InfoFrame and put up splash screen
        info=new InfoFrame(infoLabel,infoWidth,infoHeight);
        info.setBackground(Color.white);
        info.resize(mfWidth,mfHeight);
        info.setLocation(infoXPosn,infoYPosn);
        info.pack();
        info.show();
 
        //initialise MainFrame
        setBackground(Color.white);
        setFont(DefaultFont.normal);
        resize(mfWidth,mfHeight);
        setLocation(mfXPosn,mfYPosn);
 
        // setup MainFrame Layout
        GridBagLayout mfLayout=new GridBagLayout();
        GridBagConstraints gblConstraints=new GridBagConstraints();
        setLayout(mfLayout);
                
        int componentHeight; //temp variable used to hold component height
        double componentRatio; //total of all MainFrame component ratios=1.0

	//components fill their allocated areas
        gblConstraints.fill=GridBagConstraints.BOTH;
	//stretch out to max. width (for one column layout)
        gblConstraints.weightx=1.0;

        // Menubar
	// Note that the 2nd arg to new Menu(), if given, is true to
	// make a tear-off menu.
        MenuBar mfMenuBar=new MenuBar();
        Menu mfMenuFile=new Menu("File");
        Menu mfMenuWindows=new Menu("Windows");
        Menu mfMenuGraph=new Menu("Graph");
	mfMenuGraphCOA=new COAMenu("Show COA status", true);
	Menu mfMenuGraphNodes=new Menu("Show process step", true);
	Menu mfMenuMatrix=new Menu("Matrix");
	Menu mfMenuTest=new Menu("Test");
	Menu mfMenuTestSimulations=new Menu("Replay log file");
	Menu mfMenuTestStates=new Menu("Show demonstration state", true);
	Menu mfHelpMenu=new Menu("Help");

        MenuItem mfMenuFileReset=new MenuItem("Reset");
        MenuItem mfMenuFileRefresh=new MenuItem("Refresh");
        MenuItem mfMenuFileQuit=new MenuItem("Quit");

        MenuItem mfMenuGraph0=new MenuItem("Top level");
        MenuItem mfMenuGraph1=new MenuItem("Develop JFACC guidance");
        MenuItem mfMenuGraph2=new MenuItem("Develop COA");
        MenuItem mfMenuGraph3=new MenuItem("Create COA");
        MenuItem mfMenuGraph4=new MenuItem("Execute plan-cycle");
        MenuItem mfMenuGraph5=new MenuItem("Create support plan");
        MenuItem mfMenuGraph6=new MenuItem("Execute support-plan-cycle");
        MenuItem mfMenuGraph7=new MenuItem("Review COA");

	MenuItem mfHelpMenuIntro=new MenuItem("Intro");
	MenuItem mfHelpMenuHelp=new MenuItem("Help");
	MenuItem mfHelpMenuKey=new MenuItem("Key");

        mfMenuFile.add(mfMenuFileReset);
        mfMenuFile.add(mfMenuFileQuit);

        mfMenuWindows.add(mfMenuFileRefresh);

        mfMenuGraphNodes.add(mfMenuGraph0);
        mfMenuGraphNodes.add(mfMenuGraph1);
        mfMenuGraphNodes.add(mfMenuGraph2);
        mfMenuGraphNodes.add(mfMenuGraph3);
        mfMenuGraphNodes.add(mfMenuGraph4);
        mfMenuGraphNodes.add(mfMenuGraph5);
        mfMenuGraphNodes.add(mfMenuGraph6);
        mfMenuGraphNodes.add(mfMenuGraph7);

        // mfMenuGraphCOA.add(new MenuItem("Overall"));
        // mfMenuGraphCOA.add(new MenuItem("COA-1"));
        // mfMenuGraphCOA.add(new MenuItem("COA-2"));
        // mfMenuGraphCOA.add(new MenuItem("COA-3"));

	mfMenuGraph.add(mfMenuGraphNodes);
	mfMenuGraph.add(mfMenuGraphCOA);

	// mfMenuTest
	for (Enumeration ne = AutoTester.testNames(); ne.hasMoreElements();) {
	    String name = (String)ne.nextElement();
	    mfMenuTestSimulations.add(new MenuItem(name));
	}

        mfMenuTestStates.add(new MenuItem("State 0"));
        mfMenuTestStates.add(new MenuItem("State 1"));
        mfMenuTestStates.add(new MenuItem("State 2"));
        mfMenuTestStates.add(new MenuItem("State 3"));
        mfMenuTestStates.add(new MenuItem("State 4"));
        mfMenuTestStates.add(new MenuItem("State 5"));
        mfMenuTestStates.add(new MenuItem("State 6"));
	mfMenuTestStates.add(new MenuItem("Monitor"));

	mfMenuTest.add(mfMenuTestSimulations);
	mfMenuTest.add(mfMenuTestStates);

	mfHelpMenu.add(mfHelpMenuIntro);
	mfHelpMenu.add(mfHelpMenuHelp);
	mfHelpMenu.add(mfHelpMenuKey);

        mfMenuBar.add(mfMenuFile);
        mfMenuBar.add(mfMenuWindows);
        mfMenuBar.add(mfMenuGraph);
	mfMenuBar.add(mfMenuMatrix);
	mfMenuBar.add(mfMenuTest);
	mfMenuBar.add(mfHelpMenu);
        mfMenuBar.setHelpMenu(mfHelpMenu);
	setMenuBar(mfMenuBar);

        // Panel 1 - ViewGraph in a ScrollPane
        ScrollPane vgsp=new ScrollPane();        
        componentRatio=0.19;
        componentHeight=(int)(mfHeight*componentRatio); 
        gblConstraints.gridx=0;
        gblConstraints.gridy=0;
        gblConstraints.weighty=componentRatio;
        mfLayout.setConstraints(vgsp,gblConstraints);
        add(vgsp);        
        //now add ViewGraph to scollpane
        vg=new ViewGraph(this);  // *** changed by JL from () to (this) ***

        vgsp.add(vg);
        
        // Panel 2 - for buttons
        Panel panel2=new Panel();
        componentRatio=0.03;
        componentHeight=(int)(mfHeight*componentRatio); 
        gblConstraints.gridx=0;
        gblConstraints.gridy=1;
        gblConstraints.weighty=componentRatio;
        mfLayout.setConstraints(panel2,gblConstraints);
        panel2.add(new Button("Intro"));
        panel2.add(new Button("Help"));
        panel2.add(new Button("Key"));
        // panel2.add(new Button("Hide Info"));
        panel2.add(new Button("Reset"));
	monitorButton=new Button("Freeze");
        panel2.add(monitorButton);
        // panel2.add(new Button("Overall"));
        // panel2.add(new Button("COA-1"));
        // panel2.add(new Button("COA-2"));
        // panel2.add(new Button("COA-3"));
        panel2.add(new Button("Quit"));
        add(panel2);
        
        // Panel 3 - ViewMatrix in a ScrollPane
        ScrollPane vmsp=new ScrollPane(); 
        componentRatio=0.72; 
        componentHeight=(int)(mfHeight*componentRatio);
        gblConstraints.gridx=0;
        gblConstraints.gridy=2;
        gblConstraints.weighty=componentRatio;
        mfLayout.setConstraints(vmsp,gblConstraints);        
        add(vmsp);
        //now add ViewMatrix to scollpane
        vm=new ViewMatrix(this);
        //note mainfarme passed to ViewMatrix for container validation
        vmsp.add(vm);
        
        // Panel 4
        componentRatio=0.06;
        componentHeight=(int)(mfHeight*componentRatio);
        LogoCanvas logoCanvas=new LogoCanvas(mfWidth,componentHeight,
                      WaitImage.waitGetImage("IMG/aiai-logo-small.gif",this));                                     
        gblConstraints.gridx=0;
        gblConstraints.gridy=3;
        gblConstraints.weighty=componentRatio;
        mfLayout.setConstraints(logoCanvas,gblConstraints);
        add(logoCanvas);
        
        // Now show MainFrame
        pack();
        show();
	// Debug.noteln("vg.size = " + vg.size());
	vg.start();  // *** added by JL ***
	vm.start();  // *** added by JL ***
    }

    public void setProcessModel(ProcessModel model) {
	this.model = model;
    }
 
    public Dimension minimumSize() {
        return myDimension;
    }
 
    public Dimension preferredSize() {
        return myDimension;
    }

    private static void processGeometryParameters() {
	// Note that the variables already have their default values.
	mfWidth    = Parameters.getInt("mainWidth",  mfWidth);
	mfHeight   = Parameters.getInt("mainHeight", mfHeight);
        mfXPosn    = Parameters.getInt("mainX",      mfXPosn);
        mfYPosn    = Parameters.getInt("mainY",      mfYPosn);
        infoWidth  = Parameters.getInt("infoWidth",  infoWidth);
        infoHeight = Parameters.getInt("infoHeight", infoHeight);
        infoXPosn  = Parameters.getInt("infoX",      infoXPosn);
        infoYPosn  = Parameters.getInt("infoY",      infoYPosn);
    }
 
    public boolean handleEvent(Event ev) {
        if (ev.id==Event.WINDOW_DESTROY) {
           this.dispose();
           this.destroy(); //only if MainFrame is treated as primary interface?
           return true;
        }
        return super.handleEvent(ev);
    }
    
    public boolean action(Event ev, Object arg) {
        if ((ev.target instanceof MenuItem) || (ev.target instanceof Button)) {
            if (arg.equals("Quit")) {
                this.destroy();
                return true;
            }
	    if (arg.equals("Freeze")) {
		Debug.noteln("Freezing");
		// Change what the button says
		monitorButton.setLabel("Unfreeze");
		// and since we're making the label bigger...
		monitorButton.invalidate();
		monitorButton.getParent().validate();
		ScreenNumber = FROZEN_SCREEN;
		return true;
	    }
	    if (arg.equals("Unfreeze")) {
		Debug.noteln("Melting");
		// Change what the button says
		monitorButton.setLabel("Freeze");
		// and since we're making the label smaller...
		monitorButton.invalidate();
		monitorButton.getParent().validate();
		// Now it's like "Monitor"
                info.setInfo(info.SPLASH);                     
                info.repaint(); // will set InfoFrame as top frame
		handleMonitorButton();
		// ScreenNumber = MONITOR_SCREEN will happen eventually.
		// See handleMonitorButton() and StatusUpdate().
		return true;
	    }
	    if (arg.equals("Monitor")) {
	        Debug.noteln("monitor button pressed");
                info.setInfo(info.SPLASH);                     
                info.repaint(); // will set InfoFrame as top frame
		handleMonitorButton();
		// ScreenNumber = MONITOR_SCREEN will happen eventually.
		// See handleMonitorButton() and StatusUpdate().
		return true;
	    }
	    if (arg.equals("Top level")) {
	        vg.setGraph("top_level.txt");
	    }
	    if (arg.equals("Develop JFACC guidance")) {
	        vg.setGraph("dev_JFACC.txt");
	    }
	    if (arg.equals("Develop COA")) {
	        vg.setGraph("dev_COA.txt");
	    }
	    if (arg.equals("Create COA")) {
	        vg.setGraph("create_COA.txt");
	    }
	    if (arg.equals("Execute plan-cycle")) {
	        vg.setGraph("plan_cycle.txt");
	    }
	    if (arg.equals("Create support plan")) {
	        vg.setGraph("create_support.txt");
	    }
	    if (arg.equals("Execute support-plan-cycle")) {
	        vg.setGraph("support_cycle.txt");
	    }
	    if (arg.equals("Review COA")) {
	        vg.setGraph("review_COA.txt");
	    }
	    if (arg.equals("Overall")) {
	        vg.setCurrentCOA(0);
		return true;
	    }
	    if (arg instanceof String && ((String)arg).startsWith("COA-")) {
		// arg == COA-n
		String coa_n = (String)arg;
		Vector parts = Seq.toVector(new StringTokenizer(coa_n, "-"));
		int n;
		try { n= Integer.parseInt((String)parts.elementAt(1)); }
		catch (NumberFormatException e) {
		    Debug.warn("Bad coa int in action: " + arg);
		    return true;
		}
		vg.setCurrentCOA(n);
		return true;
	    }
            if (arg.equals("Refresh")) {
		repaint();
	        info.repaint();
		vg.repaint();
		vm.repaint();
		return true;
            }
            if (arg.equals("Reset")) {
		AutoTester.stopAll(); 	// do this 1st
	        vg.start();
		vm.vmClear();
		if (model != null) model.reset();
                info.setInfo(info.SPLASH);                     
                info.repaint(); // will set InfoFrame as top frame
		mfMenuGraphCOA.setNumberOfCOA(0);
		ScreenNumber = -1;
		// Now pretend the "Monitor" button was pressed
		handleMonitorButton();
                return true;
            }
            if (arg.equals("Intro")) {
                info.setInfo(info.INTRODUCTION);                     
                info.repaint(); // will set InfoFrame as top frame
                return true;
            }
            if (arg.equals("Help")) {
                info.setInfo(info.HELP);                     
                info.repaint();              
                return true;
            }
            if (arg.equals("Key")) {
                info.setInfo(info.COLOR_KEY);                     
                info.repaint();              
                return true;
            }
	    // *** start of section modified by JL ***
            if (arg.equals("State 0")) {
                //sample only
                vm.paintMatrix(0);
		vg.paintGraph(0);
		info.setInfo(info.DEMO_0);
		ScreenNumber = 0;
                return true;
            }
            if (arg.equals("State 1")) {
                //sample only
                vm.paintMatrix(1);
		vg.paintGraph(1);
		info.setInfo(info.DEMO_1);
		ScreenNumber = 1;
                return true;
            }
            if (arg.equals("State 2")) {
                //sample only
                vm.paintMatrix(2);
		vg.paintGraph(2);  
		info.setInfo(info.DEMO_2);
		ScreenNumber = 2;
                return true;
            }
            if (arg.equals("State 3")) {
                //sample only
                vm.paintMatrix(3);
		vg.paintGraph(3);  
		info.setInfo(info.DEMO_3);
		ScreenNumber = 3;
                return true;
            }
            if (arg.equals("State 4")) { 
                //sample only      
                vm.paintMatrix(4); 
		vg.paintGraph(4);  
		info.setInfo(info.DEMO_4);
		ScreenNumber = 4;
                return true;       
            }                      
            if (arg.equals("State 5")) { 
                //sample only      
                vm.paintMatrix(5); 
		vg.paintGraph(5);  
		info.setInfo(info.DEMO_5);
		ScreenNumber = 5;
                return true;       
            }                      
            if (arg.equals("State 6")) { 
                //sample only      
                vm.paintMatrix(6); 
		vg.paintGraph(6);  
		info.setInfo(info.DEMO_6);
		ScreenNumber = 6;
                return true;       
            }                      
	    // *** end of section modified by JL ***
	    // AutoTester menu items
	    for (Enumeration ne = AutoTester.testNames();
		 ne.hasMoreElements();) {
		String name = (String)ne.nextElement();
		if (arg.equals(name)) {
		    AutoTester.runTest(name);
		    return true;
		}
	    }
            if (arg.equals("Hide Info")) {
                info.hide();
                return true;
            }
        }
        return false;
    }
   
    public void destroy() {
        Debug.noteln("About to terminate MainFrame");
        ACP3.exit(0);
        // will not return
        Debug.assert(false, "MainFrame.destroy() should not reach here");
    }

    /**
     * Switch to showing the state from the process model.
     */
    private void handleMonitorButton() {
	// This is run by some AWT thread, so it probably shouldn't
	// take very long before returning.

	// The ProcessModel sends us information about state changes.
	// When ScreenNumber != MONITOR_SCREEN, these updates are ignored.
	// The user can then press the "Monitor" button to start seeing
	// the state maintained by the model.  When the user does that,
	// this method is called.

	// We want to stop displaying whatever we're displaying now
	// (such as the information selected by one of the numbered
	// buttons) and start displaying the model state.  But we can
	// leave things as they are until we find out what the current
	// model state is.  So we'll ask the process model to send a
	// full state description, and we'll switch to the new display
	// when that description's received (by the statusUpdate method
	// below).

	Debug.assert(model != null, "MainFrame can't reach the process model");
	model.sendFullStateDescription();

    }

    /**
     * Handle the changes sent from the ProcessModel.
     */
    public void statusUpdate(ProcessStatusUpdate up) {

	// Follow the display-or-ignore protocol.
	if (up.containsCompleteState) {
	    Debug.noteln("Switching to the monitor display.");
	    ScreenNumber = MONITOR_SCREEN;
	}
	else if (ScreenNumber != MONITOR_SCREEN)
	    return;

	vg.statusUpdate(up);
	vm.statusUpdate(up);

	mfMenuGraphCOA.setNumberOfCOA(up.numberOfCoas);

    }

}

class COAMenu extends Menu {

    COAMenu (String label) {
	super(label);
    }

    COAMenu (String label, boolean tearOff) {
	super(label, tearOff);
    }

    // Initialization
    {
	add("Overall");
    }

    synchronized void setNumberOfCOA(int n) {
	int have = getItemCount() - 1; 		// -1 for the "Overall" item
	if (n < have) {
	    for (int i = have; i > n; i--) {
		removeCOA(i);
	    }
	}
	else if (n > have) {
	    for (int i = have+1; i <= n; i++) {
		add("COA-" + i);
	    }
	}
    }

    private void removeCOA(int i) {
	// "Overall" is at index 0, so COA-i is at index i.
	remove(i);
    }

}
