/* File: ProcessStatusUpdate.java
 * Contains: The class for information sent to ProcessViewers
 * Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Created: January 1998
 * Updated: Sun Aug 23 18:01:28 1998 by Jeff Dalton
 * Copyright: (c) 1998, AIAI, University of Edinburgh
 */

package oplan.acp3;

import java.util.*;

/**
 * A ProcessStatusUpdate is sent to an implementation of the ProcessViewer
 * interface when telling it of state changes in a process.<p>
 *
 * The update always contains the current number of COAs and the
 * total number of issues for each COA, even if these numbers have
 * not changed since the previous update was sent.  The statusChanges
 * field normally contains only the changes since the last update.
 * In that case, the containsCompleteState field is false.  However,
 * if containsCompleteState is true, then statusChanges contains
 * a StatusChange object for every node that currently exists.
 *
 * @see ProcessModel
 * @see StatusChange
 * @see ProcessViewer
 */

public class ProcessStatusUpdate {

    public int numberOfCoas;
    public Vector statusChanges;
    public int[] issueCounts;
    public boolean containsCompleteState = false;

    public ProcessStatusUpdate() {
    }

}
